#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <unistd.h>

#include "chrono.h"

struct termios stored_settings;
int termios_set = 0;

void set_keypress(void) {
	struct termios new_settings;

	if (0==termios_set) {
		tcgetattr(0, &stored_settings);
		new_settings = stored_settings;
		/* Disable canonical mode, and set buffer size to 1 byte */
		new_settings.c_lflag &= (~ICANON);
		new_settings.c_cc[VTIME] = 0;
		new_settings.c_cc[VMIN] = 1;
		new_settings.c_lflag &= (~ECHO);
		tcsetattr(0, TCSANOW, &new_settings);
		termios_set = 1;
	}
	return;
}

void reset_keypress(void) {
	if (1==termios_set) {
		tcsetattr(0, TCSANOW, &stored_settings);
	}
	return;
}

/* display on the terminal */
void print_time(struct timeval tv) {
	int h, m, s, ms;
	ms = tv.tv_usec/10000;
	h = tv.tv_sec/3600;
	m = (tv.tv_sec - h*3600)/60;
	s = tv.tv_sec - h*3600 - m*60;
	printf("%02d:%02d:%02d.%02d", h, m, s, ms);
}

int main(void) {
	char c;
	int l;
	fd_set rfds;
	struct timeval tv_sel, tv, lap;
	int retval;
	int stop = 0;
	int running;

	chrono(CHR_RESET, NULL, NULL);
	printf("Press S to start the chronometer...\n");
	set_keypress();
	while (1) {
		FD_ZERO(&rfds);
		FD_SET(STDIN_FILENO, &rfds); /* stdin */
		if (!chrono(CHR_RUNNING, NULL, NULL)) {
			/* restart only when S or s is pressed */
			retval = select(1, &rfds, NULL, NULL, NULL);
		} else {
			/* timer 10 ms */
			tv_sel.tv_sec = 0;
			tv_sel.tv_usec = 10000;
			retval = select(1, &rfds, NULL, NULL, &tv_sel);
		}
		if (retval>0) { /* a key was pressed */
			l = read(STDIN_FILENO, &c, 1);
			if (1==l) {
				switch (c) {
				case 'q':
				case 'Q':
					printf(" QUIT\n");
					reset_keypress();
					exit(0);
					break;
				case 'l':
				case 'L':
					chrono(CHR_LAP, &tv, &lap);
					printf("%c", 13);
					print_time(tv);
					printf(" ");
					print_time(lap);
					printf(" LAP\n");
					break;
				case 's':
				case 'S':
					running = chrono(CHR_START_STOP, &tv, &lap);
					if (!running) {
						printf("%c", 13);
						print_time(tv);
						printf(" ");
						print_time(lap);
						printf(" STOP\n");
						stop = 1;
					}
					break;
				case 'r':
				case 'R':
					chrono(CHR_RESET, NULL, NULL);
					printf(" RESET\n");
					break;
				default:
					fprintf(stderr, "\ninvalid key %c (use s, l, q)\n", c);
				}
			} else {
				fprintf(stderr, "read error\n");
				reset_keypress();
				exit(1);
			}
		} else if (retval==0) { /* no key pressed */
			chrono(CHR_POLL, &tv, &lap);
			printf("%c", 13);
			print_time(tv);
			printf(" ");
			print_time(lap);
		} else { /* error */
			fprintf(stderr, "select error\n");
			reset_keypress();
			exit(1);
		}

		fflush(stdout);
	}
	return 0;
}
